@@ -273,13 +273,13 @@ static void DrawRegs(RegGroupType *rg, SDL_Surface *surface, uint32 *pixels, int
    if(rec->bsize != 0xFFFF)
    {
     if(rec->bsize == 4)
-     snprintf(nubuf, 256, ": %08X", rg->GetRegister(rec->name, details_ptr));
+     trio_snprintf(nubuf, 256, ": %08X", rg->GetRegister(rec->name, details_ptr));
     else if(rec->bsize == 3)
-     snprintf(nubuf, 256, ": %06X", rg->GetRegister(rec->name, details_ptr));
+     trio_snprintf(nubuf, 256, ": %06X", rg->GetRegister(rec->name, details_ptr));
     else if(rec->bsize == 2)
-     snprintf(nubuf, 256, ": %04X", rg->GetRegister(rec->name, details_ptr));
+     trio_snprintf(nubuf, 256, ": %04X", rg->GetRegister(rec->name, details_ptr));
     else if(rec->bsize == 1)
-     snprintf(nubuf, 256, ": %02X", rg->GetRegister(rec->name, details_ptr));
+     trio_snprintf(nubuf, 256, ": %02X", rg->GetRegister(rec->name, details_ptr));
     DrawTextTrans(row + prew, surface->pitch, 64, (UTF8*)nubuf, color, 0, which_font);
    }
 
@@ -314,18 +314,18 @@ static void DrawZP(SDL_Surface *surface, uint32 *pixels)
     r = 0x00;
     g = 0xE0;
     b = 0x00;
-    snprintf(tbuf, 32, "x%1X", x);
+    trio_snprintf(tbuf, 32, "x%1X", x);
    }
    else if(x == -1 || x == 16)
    {
     r = 0x00;
     g = 0xE0;
     b = 0x00;
-    snprintf(tbuf, 32, "%1Xx", y);
+    trio_snprintf(tbuf, 32, "%1Xx", y);
    }
    else
    {
-    snprintf(tbuf, 32, "%02X", zebyte);
+    trio_snprintf(tbuf, 32, "%02X", zebyte);
 
     r = 0x00;
     g = 0x00;
@@ -662,9 +662,9 @@ void Debugger_Draw(SDL_Surface *surface, SDL_Rect *rect, const SDL_Rect *screen_
   uint32 addr_color = MK_COLOR_A(0xa0, 0xa0, 0xFF, 0xFF);
 
   if(CurGame->Debugger->LogAddrBits == 32)
-   snprintf(addr_text, 256, " %08X: ", dis_A);
+   trio_snprintf(addr_text, 256, " %08X: ", dis_A);
   else
-   snprintf(addr_text, 256, " %04X: ", dis_A);
+   trio_snprintf(addr_text, 256, " %04X: ", dis_A);
   
   if(dis_A == DisAddr)
   {
@@ -733,7 +733,7 @@ void Debugger_Draw(SDL_Surface *surface, SDL_Rect *rect, const SDL_Rect *screen_
    strbuf_len = 0;
    for(unsigned int y = 0; y < btrace.size() / btrace_rows; y++)
    {
-    strbuf_len += sprintf(strbuf + strbuf_len, "%s→", btrace[btrace_index].c_str());
+    strbuf_len += trio_sprintf(strbuf + strbuf_len, "%s→", btrace[btrace_index].c_str());
     btrace_index++;
    }
 
@@ -781,13 +781,13 @@ void Debugger_Draw(SDL_Surface *surface, SDL_Rect *rect, const SDL_Rect *screen_
    ewa_mask = ((uint64)1 << ewa_bits) - 1;
 
    if(ewa_bits <= 16)
-    snprintf(tbuf, 256, "%04X: ", (ewa + y * bytes_per_row) & ewa_mask);
+    trio_snprintf(tbuf, 256, "%04X: ", (ewa + y * bytes_per_row) & ewa_mask);
    else if(ewa_bits <= 20)
-    snprintf(tbuf, 256, "%05X: ", (ewa + y * bytes_per_row) & ewa_mask);
+    trio_snprintf(tbuf, 256, "%05X: ", (ewa + y * bytes_per_row) & ewa_mask);
    else if(ewa_bits <= 24)
-    snprintf(tbuf, 256, "%06X: ", (ewa + y * bytes_per_row) & ewa_mask);
+    trio_snprintf(tbuf, 256, "%06X: ", (ewa + y * bytes_per_row) & ewa_mask);
    else
-    snprintf(tbuf, 256, "%08X: ", (ewa + y * bytes_per_row) & ewa_mask);
+    trio_snprintf(tbuf, 256, "%08X: ", (ewa + y * bytes_per_row) & ewa_mask);
 
    row += DrawTextTrans(row, surface->pitch, rect->w, (UTF8 *)tbuf, MK_COLOR_A(0xa0, 0xa0, 0xFF, 0xFF), FALSE, MDFN_FONT_5x7);
    for(int x = 0; x < bytes_per_row; x++)
@@ -805,7 +805,7 @@ void Debugger_Draw(SDL_Surface *surface, SDL_Rect *rect, const SDL_Rect *screen_
 
     if(x == 16) row += 7;
 
-    snprintf(tbuf, 256, "%02X", zebyte);
+    trio_snprintf(tbuf, 256, "%02X", zebyte);
     row += DrawTextTrans(row, surface->pitch, rect->w, (UTF8*)tbuf, bcolor, 0, 1) + 2;
    }
    DrawTextTrans(row + 5, surface->pitch, rect->w, (UTF8 *)asciistr, MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, MDFN_FONT_5x7);
@@ -905,6 +905,12 @@ void Debugger_ForceStepIfStepping(void)
   NeedStep = 2;
 }
 
+void Debugger_ForceSteppingMode(void)
+{
+ if(!InSteppingMode)
+  NeedStep = 2;
+}
+
 // Call this function from any thread:
 // Only call with w or h non-NULL in main thread
 bool Debugger_IsActive(unsigned int *w, unsigned int *h)
@@ -1355,7 +1361,7 @@ void Debugger_Event(const SDL_Event *event)
 		  {
 		   InPrompt = WatchGoto;
 		   ptext = "Watch Address";
-                   snprintf(buf, 64, "%08X", WatchLogical ? WatchAddr : WatchAddrPhys);
+                   trio_snprintf(buf, 64, "%08X", WatchLogical ? WatchAddr : WatchAddrPhys);
 	 	  }
 		  else
 		  {
@@ -1373,17 +1379,17 @@ void Debugger_Event(const SDL_Event *event)
 		    LockGameMutex(0);
 
 		    if(len == 1)
-		     snprintf(buf, 64, "%02X", RegValue);
+		     trio_snprintf(buf, 64, "%02X", RegValue);
 		    else if(len == 2)
-		     snprintf(buf, 64, "%04X", RegValue);
+		     trio_snprintf(buf, 64, "%04X", RegValue);
 		    else
-		     snprintf(buf, 64, "%08X", RegValue);
+		     trio_snprintf(buf, 64, "%08X", RegValue);
 		   }
 		   else
 		   {
 		    InPrompt = DisGoto;
 		    ptext = "Disassembly Address";
-                    snprintf(buf, 64, "%04X", DisAddr);
+                    trio_snprintf(buf, 64, "%04X", DisAddr);
 		   }
 		  }
 
