@@ -263,6 +263,8 @@ static MDFNSetting DriverSettings[] =
   { "helpenabled", gettext_noop("Enable the help screen."), MDFNST_BOOL, "1" },
 
   #ifdef WANT_DEBUGGER
+  { "debugger.autostepmode", gettext_noop("Automatically go into the debugger's step mode after a game is loaded."), MDFNST_BOOL, "0" },
+
   { "nes.debugger.memcharset", gettext_noop("Character set for the debugger's memory editor."), MDFNST_STRING, "UTF-8" },
   { "pce.debugger.memcharset", gettext_noop("Character set for the debugger's memory editor."), MDFNST_STRING, "shift_jis" },
   { "pcfx.debugger.memcharset", gettext_noop("Character set for the debugger's memory editor."), MDFNST_STRING, "shift_jis" },
@@ -1149,6 +1151,11 @@ int main(int argc, char *argv[])
 
          #ifdef WANT_DEBUGGER
          MemDebugger_Init();
+         if(MDFN_GetSettingB("debugger.autostepmode"))
+         {
+          Debugger_Toggle();
+          Debugger_ForceSteppingMode();
+         }
          #endif
         }
 	else
