@@ -820,7 +820,7 @@ uint8 gbReadMemory(uint16 address)
     if(mapperReadRAM)
       retval = mapperReadRAM(address);
     else if(mapper)
-      retval = gbMemoryMap[address>>12][address&0x0fff];
+      retval = gbMemoryMap[address>>12][address & 0x0fff & gbRamSizeMask];
   }
   else if(address < 0xfe00)
   {
