@@ -511,56 +511,65 @@ bool8 CPUReadBatteryFile(const char *filename)
 
 void CPUCleanUp()
 {
-  if(rom != NULL) {
-    free(rom);
-    rom = NULL;
-  }
+ if(rom)
+ {
+  MDFN_free(rom);
+  rom = NULL;
+ }
 
-  if(vram != NULL) {
-    free(vram);
-    vram = NULL;
-  }
+ if(vram)
+ {
+  MDFN_free(vram);
+  vram = NULL;
+ }
 
-  if(paletteRAM != NULL) {
-    free(paletteRAM);
-    paletteRAM = NULL;
-  }
-  
-  if(internalRAM != NULL) {
-    free(internalRAM);
-    internalRAM = NULL;
-  }
+ if(paletteRAM)
+ {
+  MDFN_free(paletteRAM);
+  paletteRAM = NULL;
+ }
 
-  if(workRAM != NULL) {
-    free(workRAM);
-    workRAM = NULL;
-  }
+ if(internalRAM)
+ {
+  MDFN_free(internalRAM);
+  internalRAM = NULL;
+ }
 
-  if(bios != NULL) {
-    free(bios);
-    bios = NULL;
-  }
+ if(workRAM)
+ {
+  MDFN_free(workRAM);
+  workRAM = NULL;
+ }
 
-  if(pix != NULL) {
-    free(pix);
-    pix = NULL;
-  }
+ if(bios)
+ {
+  MDFN_free(bios);
+  bios = NULL;
+ }
 
-  if(oam != NULL) {
-    free(oam);
-    oam = NULL;
-  }
+ if(pix)
+ {
+  MDFN_free(pix);
+  pix = NULL;
+ }
 
-  if(ioMem != NULL) {
-    free(ioMem);
-    ioMem = NULL;
-  }
-  
-  if(systemColorMap32)
-  {
-   free(systemColorMap32);
-   systemColorMap32 = NULL;
-  }
+ if(oam)
+ {
+  MDFN_free(oam);
+  oam = NULL;
+ }
+
+ if(ioMem)
+ {
+  MDFN_free(ioMem);
+  ioMem = NULL;
+ }
+
+ if(systemColorMap32)
+ {
+  MDFN_free(systemColorMap32);
+  systemColorMap32 = NULL;
+ }
 
  GBA_Flash_Kill();
 }
@@ -660,21 +669,30 @@ static bool TestMagic(const char *name, MDFNFILE *fp)
 
 static int Load(const char *name, MDFNFILE *fp)
 {
-  int size = 0x2000000;
-
   if(!TestMagic(name, fp))
    return(-1);
 
   layerSettings = 0xFF00;
+
+  if(!(rom = (uint8 *)MDFN_malloc(0x2000000, _("ROM"))))
+   return(0);
+
+  memset(rom, 0xFF, 0x2000000);
+
+  if(!(workRAM = (uint8 *)MDFN_calloc(1, 0x40000, _("Work RAM"))))
+  {
+   MDFN_free(rom);
+   return(0);
+  }
+
+
   if(!memcmp(fp->data, "PSF\x22", 4))
   {
-   rom = (uint8 *)malloc(0x2000000);
-   workRAM = (uint8 *)calloc(1, 0x40000);
    int t = MDFNPSF_Load(0x22, fp, &pi, gsf_load_func);
    if(!t)
    {
-    free(workRAM);
-    free(rom);
+    MDFN_free(workRAM);
+    MDFN_free(rom);
     return(0);
    }
    static UTF8 *sn;
@@ -683,74 +701,79 @@ static int Load(const char *name, MDFNFILE *fp)
   }
   else
   {
-    rom = (uint8 *)malloc(0x2000000);
-    workRAM = (uint8 *)calloc(1, 0x40000);
+   uint32 size = fp->size;
+   uint8 *whereToLoad;
 
-    uint8 *whereToLoad = rom;
+   if(cpuIsMultiBoot)
+   {
+    whereToLoad = workRAM;
+    if(size > 0x40000)
+     size = 0x40000;
+   }
+   else
+   {
+    whereToLoad = rom;
+    if(size > 0x2000000)
+     size = 0x2000000;
+   }
 
-    if(cpuIsMultiBoot)
-      whereToLoad = workRAM;
+   memcpy(whereToLoad, fp->data, size);
 
-    memcpy(whereToLoad, fp->data, fp->size);
-    md5_context md5;
-    md5.starts();
-    md5.update(fp->data, fp->size);
-    md5.finish(MDFNGameInfo->MD5);
+   md5_context md5;
+   md5.starts();
+   md5.update(fp->data, size);
+   md5.finish(MDFNGameInfo->MD5);
 
-    MDFN_printf(_("ROM:       %dKiB\n"), (fp->size + 1023) / 1024);
-    MDFN_printf(_("ROM CRC32: 0x%04x\n"), crc32(0, fp->data, fp->size));
-    MDFN_printf(_("ROM MD5:   0x%s\n"), md5_context::asciistr(MDFNGameInfo->MD5, 0).c_str());
+   MDFN_printf(_("ROM:       %dKiB\n"), (size + 1023) / 1024);
+   MDFN_printf(_("ROM CRC32: 0x%04x\n"), crc32(0, fp->data, size));
+   MDFN_printf(_("ROM MD5:   0x%s\n"), md5_context::asciistr(MDFNGameInfo->MD5, 0).c_str());
 
-    uint16 *temp = (uint16 *)(rom+((size+1)&~1));
-    int i;
-    for(i = (size+1)&~1; i < 0x2000000; i+=2) {
-      WRITE16LE(temp, (i >> 1) & 0xFFFF);
-      temp++;
-    }
-  }
-  bios = (uint8 *)calloc(1,0x4000);
-  if(bios == NULL) {
-   // systemMessage(MSG_OUT_OF_MEMORY, N_("Failed to allocate memory for %s"),
-   //               "BIOS");
-    CPUCleanUp();
-    return 0;
-  }
-  internalRAM = (uint8 *)calloc(1,0x8000);
-  if(internalRAM == NULL) {
-  //  systemMessage(MSG_OUT_OF_MEMORY, N_("Failed to allocate memory for %s"),
-   //               "IRAM");
-    CPUCleanUp();
-    return 0;
-  }
-  paletteRAM = (uint8 *)calloc(1,0x400);
-  if(paletteRAM == NULL) {
-   // systemMessage(MSG_OUT_OF_MEMORY, N_("Failed to allocate memory for %s"),
-   //               "PRAM");
-    CPUCleanUp();
-    return 0;
-  }
-  vram = (uint8 *)calloc(1, 0x20000);
-  if(vram == NULL) {
-    //systemMessage(MSG_OUT_OF_MEMORY, N_("Failed to allocate memory for %s"),
-    //              "VRAM");
-    CPUCleanUp();
-    return 0;
-  }
-  oam = (uint8 *)calloc(1, 0x400);
-  if(oam == NULL) {
-    //systemMessage(MSG_OUT_OF_MEMORY, N_("Failed to allocate memory for %s"),
-    //              "OAM");
-    CPUCleanUp();
-    return 0;
+   uint16 *temp = (uint16 *)(rom+((size+1)&~1));
+   int i;
+
+   for(i = (size+1)&~1; i < 0x2000000; i+=2)
+   {
+    WRITE16LE(temp, (i >> 1) & 0xFFFF);
+    temp++;
+   }
   }
 
-  ioMem = (uint8 *)calloc(1, 0x400);
-  if(ioMem == NULL) {
-    //systemMessage(MSG_OUT_OF_MEMORY, N_("Failed to allocate memory for %s"),
-     //             "IO");
-    CPUCleanUp();
-    return 0;
+  if(!(bios = (uint8 *)MDFN_calloc(1, 0x4000, _("BIOS ROM"))))
+  {
+   CPUCleanUp();
+   return 0;
   }
+
+  if(!(internalRAM = (uint8 *)MDFN_calloc(1, 0x8000, _("Internal RAM"))))
+  {
+   CPUCleanUp();
+   return 0;
+  }
+
+  if(!(paletteRAM = (uint8 *)MDFN_calloc(1, 0x400, _("Palette RAM"))))
+  {
+   CPUCleanUp();
+   return 0;
+  }
+
+  if(!(vram = (uint8 *)MDFN_calloc(1, 0x20000, _("VRAM"))))
+  {
+   CPUCleanUp();
+   return 0;
+  }
+
+  if(!(oam = (uint8 *)MDFN_calloc(1, 0x400, _("OAM"))))
+  {
+   CPUCleanUp();
+   return 0;
+  }
+
+  if(!(ioMem = (uint8 *)MDFN_calloc(1, 0x400, _("IO"))))
+  {
+   CPUCleanUp();
+   return 0;
+  }
+
   if(!(systemColorMap32 = (uint32*)MDFN_malloc(65536 * sizeof(uint32), _("GBA Color Map"))))
   {
    CPUCleanUp();
@@ -815,7 +838,7 @@ static int Load(const char *name, MDFNFILE *fp)
     if(gzread(gp, CustomColorMap, 32768 * 3) != 32768 * 3)
     {
      MDFN_printf("Error\n");
-     free(CustomColorMap);
+     MDFN_free(CustomColorMap);
      CustomColorMap = NULL;
     }
     else
