@@ -23,14 +23,14 @@
 #include "Sound.h"
 
 #include <blip/Blip_Buffer.h>
-#include "../gb/sound/Gb_Apu.h"
+#include "Gba_Apu.h"
 #include <blip/Stereo_Buffer.h>
 
 typedef Blip_Synth<blip_good_quality, 0xFF * 2> Synth;
 
 static Synth synth;
-static Gb_Apu gb_apu;
-static Stereo_Buffer gb_buf;
+static Gba_Apu gba_apu;
+static Stereo_Buffer gba_buf;
 
 extern uint32 soundTS;
 
@@ -75,116 +75,116 @@ int MDFNGBASOUND_StateAction(StateMem *sm, int load, int data_only)
   {soundDSFifoB, sizeof(soundDSFifoB), "soundDSFifoB" },
   SFVAR(soundDSBValue),
 
- {&gb_apu.regs, sizeof(gb_apu.regs), "REGS"},
+ {&gba_apu.regs, sizeof(gba_apu.regs), "REGS"},
 
- {&gb_apu.square1.phase, sizeof(int) | MDFNSTATE_RLSB, "s1PH" },
- {&gb_apu.square1.duty, sizeof(int) | MDFNSTATE_RLSB, "s1DU" },
+ {&gba_apu.square1.phase, sizeof(int) | MDFNSTATE_RLSB, "s1PH" },
+ {&gba_apu.square1.duty, sizeof(int) | MDFNSTATE_RLSB, "s1DU" },
 
- {&gb_apu.square1.sweep_period, sizeof(int) | MDFNSTATE_RLSB, "s1SP" },
- {&gb_apu.square1.sweep_delay, sizeof(int) | MDFNSTATE_RLSB, "s1SweepDelay" },
- {&gb_apu.square1.sweep_shift, sizeof(int) | MDFNSTATE_RLSB, "s1SS" },
- {&gb_apu.square1.sweep_dir, sizeof(int) | MDFNSTATE_RLSB, "s1SweepDir" },
- {&gb_apu.square1.sweep_freq, sizeof(int) | MDFNSTATE_RLSB, "s1SF" },
+ {&gba_apu.square1.sweep_period, sizeof(int) | MDFNSTATE_RLSB, "s1SP" },
+ {&gba_apu.square1.sweep_delay, sizeof(int) | MDFNSTATE_RLSB, "s1SweepDelay" },
+ {&gba_apu.square1.sweep_shift, sizeof(int) | MDFNSTATE_RLSB, "s1SS" },
+ {&gba_apu.square1.sweep_dir, sizeof(int) | MDFNSTATE_RLSB, "s1SweepDir" },
+ {&gba_apu.square1.sweep_freq, sizeof(int) | MDFNSTATE_RLSB, "s1SF" },
 
- {&gb_apu.square1.env_period, sizeof(int) | MDFNSTATE_RLSB, "s1EP" },
- {&gb_apu.square1.env_dir, sizeof(int) | MDFNSTATE_RLSB, "s1ED" },
- {&gb_apu.square1.env_delay, sizeof(int) | MDFNSTATE_RLSB, "s1Ed" },
+ {&gba_apu.square1.env_period, sizeof(int) | MDFNSTATE_RLSB, "s1EP" },
+ {&gba_apu.square1.env_dir, sizeof(int) | MDFNSTATE_RLSB, "s1ED" },
+ {&gba_apu.square1.env_delay, sizeof(int) | MDFNSTATE_RLSB, "s1Ed" },
 
- {&gb_apu.square1.new_env_period, sizeof(int) | MDFNSTATE_RLSB, "s1NP" },
- {&gb_apu.square1.new_env_dir, sizeof(int) | MDFNSTATE_RLSB, "s1ND" },
- {&gb_apu.square1.new_volume, sizeof(int) | MDFNSTATE_RLSB, "s1NV" },
+ {&gba_apu.square1.new_env_period, sizeof(int) | MDFNSTATE_RLSB, "s1NP" },
+ {&gba_apu.square1.new_env_dir, sizeof(int) | MDFNSTATE_RLSB, "s1ND" },
+ {&gba_apu.square1.new_volume, sizeof(int) | MDFNSTATE_RLSB, "s1NV" },
 
- {&gb_apu.square1.period, sizeof(int) | MDFNSTATE_RLSB, "s1OP" },
- {&gb_apu.square1.volume, sizeof(int) | MDFNSTATE_RLSB, "s1OV" },
- {&gb_apu.square1.frequency, sizeof(int) | MDFNSTATE_RLSB, "s1OF" },
- {&gb_apu.square1.length, sizeof(int) | MDFNSTATE_RLSB, "s1OL" },
- {&gb_apu.square1.new_length, sizeof(int) | MDFNSTATE_RLSB, "s1Ol" },
- {&gb_apu.square1.ME, sizeof(int) | MDFNSTATE_RLSB, "s1OM" },
- {&gb_apu.square1.enabled, sizeof(bool8) | MDFNSTATE_RLSB, "s1OE" },
- {&gb_apu.square1.length_enabled, sizeof(bool8) | MDFNSTATE_RLSB, "s1Oe" },
- {&gb_apu.square1.output_select, sizeof(int) | MDFNSTATE_RLSB, "s1OS" },
+ {&gba_apu.square1.period, sizeof(int) | MDFNSTATE_RLSB, "s1OP" },
+ {&gba_apu.square1.volume, sizeof(int) | MDFNSTATE_RLSB, "s1OV" },
+ {&gba_apu.square1.frequency, sizeof(int) | MDFNSTATE_RLSB, "s1OF" },
+ {&gba_apu.square1.length, sizeof(int) | MDFNSTATE_RLSB, "s1OL" },
+ {&gba_apu.square1.new_length, sizeof(int) | MDFNSTATE_RLSB, "s1Ol" },
+ {&gba_apu.square1.ME, sizeof(int) | MDFNSTATE_RLSB, "s1OM" },
+ {&gba_apu.square1.enabled, sizeof(bool8) | MDFNSTATE_RLSB, "s1OE" },
+ {&gba_apu.square1.length_enabled, sizeof(bool8) | MDFNSTATE_RLSB, "s1Oe" },
+ {&gba_apu.square1.output_select, sizeof(int) | MDFNSTATE_RLSB, "s1OS" },
 
 
- {&gb_apu.square2.phase, sizeof(int) | MDFNSTATE_RLSB, "s2PH" },
- {&gb_apu.square2.duty, sizeof(int) | MDFNSTATE_RLSB, "s2DU" },
+ {&gba_apu.square2.phase, sizeof(int) | MDFNSTATE_RLSB, "s2PH" },
+ {&gba_apu.square2.duty, sizeof(int) | MDFNSTATE_RLSB, "s2DU" },
 
- {&gb_apu.square2.sweep_period, sizeof(int) | MDFNSTATE_RLSB, "s2SP" },
- {&gb_apu.square2.sweep_delay, sizeof(int) | MDFNSTATE_RLSB, "s2SweepDelay" },
- {&gb_apu.square2.sweep_shift, sizeof(int) | MDFNSTATE_RLSB, "s2SS" },
- {&gb_apu.square2.sweep_dir, sizeof(int) | MDFNSTATE_RLSB, "s2SweepDir" },
- {&gb_apu.square2.sweep_freq, sizeof(int) | MDFNSTATE_RLSB, "s2SF" },
- {&gb_apu.square2.env_period, sizeof(int) | MDFNSTATE_RLSB, "s2EP" },
- {&gb_apu.square2.env_dir, sizeof(int) | MDFNSTATE_RLSB, "s2ED" },
- {&gb_apu.square2.env_delay, sizeof(int) | MDFNSTATE_RLSB, "s2Ed" },
+ {&gba_apu.square2.sweep_period, sizeof(int) | MDFNSTATE_RLSB, "s2SP" },
+ {&gba_apu.square2.sweep_delay, sizeof(int) | MDFNSTATE_RLSB, "s2SweepDelay" },
+ {&gba_apu.square2.sweep_shift, sizeof(int) | MDFNSTATE_RLSB, "s2SS" },
+ {&gba_apu.square2.sweep_dir, sizeof(int) | MDFNSTATE_RLSB, "s2SweepDir" },
+ {&gba_apu.square2.sweep_freq, sizeof(int) | MDFNSTATE_RLSB, "s2SF" },
+ {&gba_apu.square2.env_period, sizeof(int) | MDFNSTATE_RLSB, "s2EP" },
+ {&gba_apu.square2.env_dir, sizeof(int) | MDFNSTATE_RLSB, "s2ED" },
+ {&gba_apu.square2.env_delay, sizeof(int) | MDFNSTATE_RLSB, "s2Ed" },
 
- {&gb_apu.square2.new_env_period, sizeof(int) | MDFNSTATE_RLSB, "s2NP" },
- {&gb_apu.square2.new_env_dir, sizeof(int) | MDFNSTATE_RLSB, "s2ND" },
- {&gb_apu.square2.new_volume, sizeof(int) | MDFNSTATE_RLSB, "s2NV" },
+ {&gba_apu.square2.new_env_period, sizeof(int) | MDFNSTATE_RLSB, "s2NP" },
+ {&gba_apu.square2.new_env_dir, sizeof(int) | MDFNSTATE_RLSB, "s2ND" },
+ {&gba_apu.square2.new_volume, sizeof(int) | MDFNSTATE_RLSB, "s2NV" },
 
- {&gb_apu.square2.period, sizeof(int) | MDFNSTATE_RLSB, "s2OP" },
- {&gb_apu.square2.volume, sizeof(int) | MDFNSTATE_RLSB, "s2OV" },
- {&gb_apu.square2.frequency, sizeof(int) | MDFNSTATE_RLSB, "s2OF" },
- {&gb_apu.square2.length, sizeof(int) | MDFNSTATE_RLSB, "s2OL" },
- {&gb_apu.square2.new_length, sizeof(int) | MDFNSTATE_RLSB, "s2Ol" },
- {&gb_apu.square2.ME, sizeof(int) | MDFNSTATE_RLSB, "s2OM" },
- {&gb_apu.square2.enabled, sizeof(bool8) | MDFNSTATE_RLSB, "s2OE" },
- {&gb_apu.square2.length_enabled, sizeof(bool8) | MDFNSTATE_RLSB, "s2Oe" },
- {&gb_apu.square2.output_select, sizeof(int) | MDFNSTATE_RLSB, "s2OS" },
+ {&gba_apu.square2.period, sizeof(int) | MDFNSTATE_RLSB, "s2OP" },
+ {&gba_apu.square2.volume, sizeof(int) | MDFNSTATE_RLSB, "s2OV" },
+ {&gba_apu.square2.frequency, sizeof(int) | MDFNSTATE_RLSB, "s2OF" },
+ {&gba_apu.square2.length, sizeof(int) | MDFNSTATE_RLSB, "s2OL" },
+ {&gba_apu.square2.new_length, sizeof(int) | MDFNSTATE_RLSB, "s2Ol" },
+ {&gba_apu.square2.ME, sizeof(int) | MDFNSTATE_RLSB, "s2OM" },
+ {&gba_apu.square2.enabled, sizeof(bool8) | MDFNSTATE_RLSB, "s2OE" },
+ {&gba_apu.square2.length_enabled, sizeof(bool8) | MDFNSTATE_RLSB, "s2Oe" },
+ {&gba_apu.square2.output_select, sizeof(int) | MDFNSTATE_RLSB, "s2OS" },
 
 
- {&gb_apu.noise.bits, sizeof(int) | MDFNSTATE_RLSB, "sNNB" },
- {&gb_apu.noise.tap, sizeof(int) | MDFNSTATE_RLSB, "sNNT" },
+ {&gba_apu.noise.bits, sizeof(int) | MDFNSTATE_RLSB, "sNNB" },
+ {&gba_apu.noise.tap, sizeof(int) | MDFNSTATE_RLSB, "sNNT" },
 
- {&gb_apu.noise.env_period, sizeof(int) | MDFNSTATE_RLSB, "sNEP" },
- {&gb_apu.noise.env_dir, sizeof(int) | MDFNSTATE_RLSB, "sNED" },
- {&gb_apu.noise.env_delay, sizeof(int) | MDFNSTATE_RLSB, "sNEd" },
+ {&gba_apu.noise.env_period, sizeof(int) | MDFNSTATE_RLSB, "sNEP" },
+ {&gba_apu.noise.env_dir, sizeof(int) | MDFNSTATE_RLSB, "sNED" },
+ {&gba_apu.noise.env_delay, sizeof(int) | MDFNSTATE_RLSB, "sNEd" },
 
- {&gb_apu.noise.new_env_period, sizeof(int) | MDFNSTATE_RLSB, "sNNP" },
- {&gb_apu.noise.new_env_dir, sizeof(int) | MDFNSTATE_RLSB, "sNND" },
- {&gb_apu.noise.new_volume, sizeof(int) | MDFNSTATE_RLSB, "sNNV" },
+ {&gba_apu.noise.new_env_period, sizeof(int) | MDFNSTATE_RLSB, "sNNP" },
+ {&gba_apu.noise.new_env_dir, sizeof(int) | MDFNSTATE_RLSB, "sNND" },
+ {&gba_apu.noise.new_volume, sizeof(int) | MDFNSTATE_RLSB, "sNNV" },
 
- {&gb_apu.noise.period, sizeof(int) | MDFNSTATE_RLSB, "sNOP" },
- {&gb_apu.noise.volume, sizeof(int) | MDFNSTATE_RLSB, "sNOV" },
- {&gb_apu.noise.frequency, sizeof(int) | MDFNSTATE_RLSB, "sNOF" },
- {&gb_apu.noise.length, sizeof(int) | MDFNSTATE_RLSB, "sNOL" },
- {&gb_apu.noise.new_length, sizeof(int) | MDFNSTATE_RLSB, "sNOl" },
- {&gb_apu.noise.ME, sizeof(int) | MDFNSTATE_RLSB, "sNOM" },
- {&gb_apu.noise.enabled, sizeof(bool8) | MDFNSTATE_RLSB, "sNOE" },
- {&gb_apu.noise.length_enabled, sizeof(bool8) | MDFNSTATE_RLSB, "sNOe" },
+ {&gba_apu.noise.period, sizeof(int) | MDFNSTATE_RLSB, "sNOP" },
+ {&gba_apu.noise.volume, sizeof(int) | MDFNSTATE_RLSB, "sNOV" },
+ {&gba_apu.noise.frequency, sizeof(int) | MDFNSTATE_RLSB, "sNOF" },
+ {&gba_apu.noise.length, sizeof(int) | MDFNSTATE_RLSB, "sNOL" },
+ {&gba_apu.noise.new_length, sizeof(int) | MDFNSTATE_RLSB, "sNOl" },
+ {&gba_apu.noise.ME, sizeof(int) | MDFNSTATE_RLSB, "sNOM" },
+ {&gba_apu.noise.enabled, sizeof(bool8) | MDFNSTATE_RLSB, "sNOE" },
+ {&gba_apu.noise.length_enabled, sizeof(bool8) | MDFNSTATE_RLSB, "sNOe" },
 
 
- {&gb_apu.wave.volume_shift, sizeof(int) | MDFNSTATE_RLSB, "sWWS" },
- {&gb_apu.wave.wave_pos, sizeof(int) | MDFNSTATE_RLSB, "sWWP" },
- {gb_apu.wave.wave, 32, "sWWW" },
+ {&gba_apu.wave.volume_shift, sizeof(int) | MDFNSTATE_RLSB, "sWWS" },
+ {&gba_apu.wave.wave_pos, sizeof(int) | MDFNSTATE_RLSB, "sWWP" },
+ {gba_apu.wave.wave, 32 * 2, "WaveData" },
 
- {&gb_apu.wave.period, sizeof(int) | MDFNSTATE_RLSB, "sWOP" },
- {&gb_apu.wave.volume, sizeof(int) | MDFNSTATE_RLSB, "sWOV" },
- {&gb_apu.wave.frequency, sizeof(int) | MDFNSTATE_RLSB, "sWOF" },
- {&gb_apu.wave.length, sizeof(int) | MDFNSTATE_RLSB, "sWOL" },
- {&gb_apu.wave.new_length, sizeof(int) | MDFNSTATE_RLSB, "sWOl" },
- {&gb_apu.wave.ME, sizeof(int) | MDFNSTATE_RLSB, "sWOM" },
- {&gb_apu.wave.enabled, sizeof(bool8) | MDFNSTATE_RLSB, "sWOE" },
- {&gb_apu.wave.length_enabled, sizeof(bool8) | MDFNSTATE_RLSB, "sWOe" },
+ {&gba_apu.wave.period, sizeof(int) | MDFNSTATE_RLSB, "sWOP" },
+ {&gba_apu.wave.volume, sizeof(int) | MDFNSTATE_RLSB, "sWOV" },
+ {&gba_apu.wave.frequency, sizeof(int) | MDFNSTATE_RLSB, "sWOF" },
+ {&gba_apu.wave.length, sizeof(int) | MDFNSTATE_RLSB, "sWOL" },
+ {&gba_apu.wave.new_length, sizeof(int) | MDFNSTATE_RLSB, "sWOl" },
+ {&gba_apu.wave.ME, sizeof(int) | MDFNSTATE_RLSB, "sWOM" },
+ {&gba_apu.wave.enabled, sizeof(bool8) | MDFNSTATE_RLSB, "sWOE" },
+ {&gba_apu.wave.length_enabled, sizeof(bool8) | MDFNSTATE_RLSB, "sWOe" },
  SFEND
  };
 
  int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "SND");
  if(load)
-  gb_apu.dirty();
+  gba_apu.dirty();
  return(ret);
 }
 
 uint8 soundRead(uint32 address)
 {
  if(address == 0x80)
-  return(gb_apu.read_register(soundTS, 0xff24));
+  return(gba_apu.read_register(soundTS, 0xff24));
  else if(address == 0x81)
-  return(gb_apu.read_register(soundTS, 0xff25));
+  return(gba_apu.read_register(soundTS, 0xff25));
  else if(address == 0x84)
-  return(gb_apu.read_register(soundTS, 0xff26));
+  return(gba_apu.read_register(soundTS, 0xff26));
  else if(address >= 0x90 && address <= 0x9f)
  {
-  return(gb_apu.read_register(soundTS, address - 0x90 + 0xff30));
+  return(gba_apu.read_register(soundTS, address - 0x90 + 0xff30));
  }
  return(ioMem[address]);
 }
@@ -251,7 +251,7 @@ void soundEvent(uint32 address, uint8 data)
  }
  ioMem[origa] = data;
 
- gb_apu.write_register(soundTS, address, data);
+ gba_apu.write_register(soundTS, address, data);
 }
 
 void soundEvent(uint32 address, uint16 data)
@@ -309,8 +309,8 @@ void soundEvent(uint32 address, uint16 data)
   case 0x9c:
   case 0x9e:
     //printf("Yay: %04x: %04x\n", 0xFF30 + (address & 0xF), data);
-    gb_apu.write_register(soundTS, 0xFF30 + (address & 0xF), data & 0xFF);
-    gb_apu.write_register(soundTS, 0xFF30 + (address & 0xF) + 1, data >> 8);
+    gba_apu.write_register(soundTS, 0xFF30 + (address & 0xF), data & 0xFF);
+    gba_apu.write_register(soundTS, 0xFF30 + (address & 0xF) + 1, data >> 8);
     //*((uint16 *)&sound3WaveRam[(sound3Bank*0x10)^0x10+(address&14)]) = data;
     //*((uint16 *)&ioMem[address]) = data;    
     break;    
@@ -423,9 +423,6 @@ static inline void soundMix(int &left, int &right)
 static int lleft = 0, lright = 0;
 static bool forcemono;
 
-#include <stdlib.h>
-#include <assert.h>
-
 void soundLick()
 {
  int left, right;
@@ -440,42 +437,42 @@ void soundLick()
   left = right = tmp;
 
   if(left != lleft)
-   synth.offset_inline(soundTS, left - lleft, gb_buf.left());
+   synth.offset_inline(soundTS, left - lleft, gba_buf.left());
  }
  else
  {
   if(left != lleft)
-   synth.offset_inline(soundTS, left - lleft, gb_buf.left());
+   synth.offset_inline(soundTS, left - lleft, gba_buf.left());
 
   if(right != lright)
-   synth.offset_inline(soundTS, right - lright, gb_buf.right());
+   synth.offset_inline(soundTS, right - lright, gba_buf.right());
  }
 
  lleft = left;
  lright = right;
 }
 
-int16 *MDFNGBASOUND_Flush(int32 *len, int needrew)
+int16 *MDFNGBASOUND_Flush(int32 *len)
 {
  static int16 yaybuf[8000];
- int love2, x;
+ int love2;
 
  int ratio = ioMem[0x82] & 3;
  int rat_table[4] = { 2, 1, 0, 2 };
  
- gb_apu.volume(0.30 * (double)(4 >> rat_table[ratio]) / 4);
+ gba_apu.volume(0.30 * (double)(4 >> rat_table[ratio]) / 4);
 
- gb_apu.end_frame(soundTS);
+ gba_apu.end_frame(soundTS);
 
  if(forcemono)
  {
-  gb_buf.left()->end_frame(soundTS);
-  love2 = gb_buf.left()->read_samples(yaybuf, 8000);
+  gba_buf.left()->end_frame(soundTS);
+  love2 = gba_buf.left()->read_samples(yaybuf, 8000);
  }
  else
  {
-  gb_buf.end_frame(soundTS);
-  love2 = gb_buf.read_samples(yaybuf, 8000);
+  gba_buf.end_frame(soundTS);
+  love2 = gba_buf.read_samples(yaybuf, 8000);
  }
 
  soundTS = 0;
@@ -486,16 +483,6 @@ int16 *MDFNGBASOUND_Flush(int32 *len, int needrew)
   return(NULL);
  }
 
- if(needrew)
- {
-  if(forcemono)
-   for(x=0;x<love2;x++)
-    yaybuf[love2 - x - 1] = yaybuf[x];
-  else
-   for(x=0;x<love2;x++)
-    yaybuf[love2 - (x&~1) - ((x&1) ^ 1) - 1] = yaybuf[x];
- }
-
  if(!forcemono)
   love2 /= 2;
 
@@ -505,21 +492,21 @@ int16 *MDFNGBASOUND_Flush(int32 *len, int needrew)
 
 void MDFNGBASOUND_Init(bool WantMono)
 {
- gb_buf.set_sample_rate(FSettings.SndRate?FSettings.SndRate:44100, 60);
- gb_buf.clock_rate((long)(4194304 * 4 * FSettings.soundmultiplier));
+ gba_buf.set_sample_rate(FSettings.SndRate?FSettings.SndRate:44100, 60);
+ gba_buf.clock_rate((long)(4194304 * 4 * FSettings.soundmultiplier));
 
  forcemono = WantMono;
 
  if(forcemono)
-  gb_apu.output(gb_buf.left(), NULL, NULL);
+  gba_apu.output(gba_buf.left(), NULL, NULL);
  else
-  gb_apu.output(gb_buf.center(), gb_buf.left(), gb_buf.right());
+  gba_apu.output(gba_buf.center(), gba_buf.left(), gba_buf.right());
 
- gb_apu.setgba(1);
- gb_apu.volume(0.30);
+ gba_apu.setgba(1);
+ gba_apu.volume(0.30);
 
  synth.volume(0.60); //0.25);
- gb_buf.bass_freq(20);
+ gba_buf.bass_freq(20);
 }
 
 void soundShutdown()
@@ -546,7 +533,7 @@ bool8 soundInit()
 
 void MDFNGBA_SetSoundMultiplier(double multiplier)
 {
-        gb_buf.clock_rate((long)(4194304 * 4 * multiplier));
+        gba_buf.clock_rate((long)(4194304 * 4 * multiplier));
 }
 
 void MDFNGBA_SetSoundVolume(uint32 volume)
@@ -555,5 +542,5 @@ void MDFNGBA_SetSoundVolume(uint32 volume)
 }
 void MDFNGBA_Sound(int rate)
 {
-        gb_buf.set_sample_rate(rate?rate:44100, 60);
+        gba_buf.set_sample_rate(rate?rate:44100, 60);
 }
