@@ -54,8 +54,6 @@
 //#include <crtdbg.h>
 //#define	TRACE_SYSTEM
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include "system.h"
 
@@ -76,31 +74,31 @@ CSystem::CSystem(uint8 *filememory, int32 filesize)
 {
 	mFileType=HANDY_FILETYPE_LNX;
 
-	// Now try and determine the filetype we have opened
-	if(filesize)
-	{
-		char clip[11];
-		memcpy(clip,filememory,11);
-		clip[4]=0;
-		clip[10]=0;
+	if(filesize < 11) throw(-1);
 
-		if(!strcmp(&clip[6],"BS93")) mFileType=HANDY_FILETYPE_HOMEBREW;
-		else if(!strcmp(&clip[0],"LYNX")) mFileType=HANDY_FILETYPE_LNX;
-		else
-		{
-			throw(-1);
-			//CLynxException lynxerr;
-			//delete filememory;
-			//mFileType=HANDY_FILETYPE_ILLEGAL;
-			//lynxerr.Message() << "Handy Error: File format invalid!";
-			//lynxerr.Description()
-			//	<< "The image you selected was not a recognised game cartridge format." << endl
-			//	<< "(see the Handy User Guide for more information).";
-			//throw(lynxerr);
-		}
+	char clip[11];
+	memcpy(clip,filememory,11);
+	clip[4]=0;
+	clip[10]=0;
+
+	if(!strcmp(&clip[6],"BS93")) mFileType=HANDY_FILETYPE_HOMEBREW;
+	else if(!strcmp(&clip[0],"LYNX")) mFileType=HANDY_FILETYPE_LNX;
+	else
+	{
+		throw(-1);
+		//CLynxException lynxerr;
+		//delete filememory;
+		//mFileType=HANDY_FILETYPE_ILLEGAL;
+		//lynxerr.Message() << "Handy Error: File format invalid!";
+		//lynxerr.Description()
+		//	<< "The image you selected was not a recognised game cartridge format." << endl
+		//	<< "(see the Handy User Guide for more information).";
+		//throw(lynxerr);
 	}
-	
-// Create the system objects that we'll use
+
+	MDFNMP_Init(65536, 1);
+
+	// Create the system objects that we'll use
 
 	// Attempt to load the cartridge errors caught above here...
 
@@ -197,7 +195,6 @@ static int Load(const char *name, MDFNFILE *fp)
 {
  try
  {
-  MDFNMP_Init(65536, 1);
   lynxie = new CSystem(fp->data, fp->size);
  }
  catch(int i)
@@ -251,26 +248,17 @@ static void CloseGame(void)
   delete lynxie;
 }
 
-static int needrew = 0;
-static void DoRewind(void)
-{
- needrew = 1;
-}
-
 static uint8 *chee;
-static void Emulate(uint32 *pXBuf, MDFN_Rect *LineWidths, int16 **SoundBuf, int32 *SoundBufSize, int skip)
+static void Emulate(EmulateSpecStruct *espec)
 {
- MDFNGameInfo->fb = pXBuf;
+ MDFNGameInfo->fb = espec->pixels;
 
  lynxie->SetButtonData(chee[0] | (chee[1] << 8));
 
- MDFN_StateEvil(needrew);
- needrew = 0;
-
  MDFNMP_ApplyPeriodicCheats();
 
- lynxie->mMikie->mpSkipFrame = skip;
- lynxie->mMikie->mpDisplayCurrent = (uint8 *)pXBuf;
+ lynxie->mMikie->mpSkipFrame = espec->skip;
+ lynxie->mMikie->mpDisplayCurrent = (uint8 *)espec->pixels;
  lynxie->mMikie->startTS = gSystemCycleCount;
 
  while(lynxie->mMikie->mpDisplayCurrent && (gSystemCycleCount - lynxie->mMikie->startTS) < 700000)
@@ -288,13 +276,13 @@ static void Emulate(uint32 *pXBuf, MDFN_Rect *LineWidths, int16 **SoundBuf, int3
   love = lynxie->mMikie->mikbuf.read_samples(yaybuf, 8000);
 
   //printf("%d %d\n",love, gSystemCycleCount - lynxie->mMikie->startTS);
-  *SoundBufSize = love;
-  *SoundBuf = yaybuf;
+  *(espec->SoundBufSize) = love;
+  *(espec->SoundBuf) = yaybuf;
  }
  else
  {
-  *SoundBufSize = 0;
-  *SoundBuf = NULL;
+  *(espec->SoundBufSize) = 0;
+  *(espec->SoundBuf) = NULL;
  }
 }
 
@@ -436,7 +424,6 @@ MDFNGI EmulatedLynx =
  NULL,
  NULL,
  StateAction,
- DoRewind,
  Emulate,
  SetPixelFormat,
  SetInput,
