@@ -15,13 +15,11 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <stdio.h>
-#include <stdlib.h>
+#include "mednafen.h"
+
 #include <string.h>
 #include <vector>
 
-#include "types.h"
-#include "mednafen.h"
 #include "driver.h"
 #include "state.h"
 #include "general.h"
