@@ -18,8 +18,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 
 #include "nes.h"
@@ -54,8 +52,9 @@ static DECLFW(FDSRAMWrite);
 static void FDSInit(void);
 static void FDSFix(int a);
 
-#define FDSRAM GameMemBlock
-#define CHRRAM   (GameMemBlock+32768)
+
+static uint8 *FDSRAM = NULL;
+static uint8 *CHRRAM = NULL;
 
 static uint8 FDSRegs[6];
 static int32 IRQLatch,IRQCount;
@@ -82,7 +81,7 @@ static int FDS_StateAction(StateMem *sm, int load, int data_only);
 
 static void RenderSoundHQ(void);
 
-void MDFN_FDSInsert(int oride)
+int FDS_DiskInsert(int oride)
 {
 	if(InDisk==255)
         {
@@ -94,22 +93,25 @@ void MDFN_FDSInsert(int oride)
          MDFN_DispMessage((UTF8 *)_("Disk %1$d Side %2$s Ejected"),SelectDisk>>1,(SelectDisk&1)?"B":"A");
          InDisk=255;
         }
+	return(TRUE);
 }
 
-void MDFN_FDSEject(void)
+int FDS_DiskEject(void)
 {
 	InDisk=255;
+	return(TRUE);
 }
 
-void MDFN_FDSSelect(void)
+int FDS_DiskSelect(void)
 {
 	if(InDisk!=255)
         {
          MDFN_DispMessage((UTF8 *)_("Eject disk before selecting."));
-	 return;
+	 return(FALSE);
         }
         SelectDisk=((SelectDisk+1)%TotalSides)&3;
         MDFN_DispMessage((UTF8 *)_("Disk %1$d Side %2$s Selected"),SelectDisk>>1,(SelectDisk&1)?"B":"A");
+	return(TRUE);
 }
 
 static void FDSFix(int a)
@@ -532,7 +534,7 @@ static int SubLoad(MDFNFILE *fp)
    MDFN_fseek(fp,0,SEEK_SET);
   }
   else
-   return(-1);
+   return(FALSE);
  } 
  else
   TotalSides=header[4];
@@ -544,7 +546,7 @@ static int SubLoad(MDFNFILE *fp)
 
  for(x=0;x<TotalSides;x++)
  {
-  diskdata[x]=(uint8 *)MDFN_malloc(65500);
+  diskdata[x]=(uint8 *)MDFN_malloc(65500, _("FDS Disk Data"));
   if(!diskdata[x])
   {
    unsigned int zol;
@@ -633,7 +635,7 @@ static int FDS_StateAction(StateMem *sm, int load, int data_only)
 typedef struct
 {
  uint8 code;
- char *name;
+ const char *name;
 } CodeNameCore;
 
 static const char* GetManName(uint8 code)
@@ -796,16 +798,26 @@ static const char* GetManName(uint8 code)
  return(_("Unknown"));
 }
 
-int FDSLoad(const char *name, MDFNFILE *fp, NESGameType *gt)
+bool FDS_TestMagic(const char *name, MDFNFILE *fp)
+{
+ if(fp->size < 16)
+  return(FALSE);
+
+ if(memcmp(fp->data, "FDS\x1a",4) && memcmp(fp->data+1,"*NINTENDO-HVC*", 14))
+  return(FALSE);
+
+ return(TRUE);
+}
+
+
+bool FDSLoad(const char *name, MDFNFILE *fp, NESGameType *gt)
 {
  MDFNFILE *zp;
- int t;
 
  MDFN_fseek(fp,0,SEEK_SET);
 
- t = SubLoad(fp);
-
- if(t < 1) return(t);
+ if(!SubLoad(fp))
+  return(FALSE);
    
  if(!(zp=MDFN_fopen(MDFN_MakeFName(MDFNMKF_FDSROM,0,0).c_str(),NULL, "rb", NULL)))  
  {
@@ -840,6 +852,16 @@ int FDSLoad(const char *name, MDFNFILE *fp, NESGameType *gt)
 
  MDFN_fclose(zp);
 
+ if(!(FDSRAM = (uint8*)MDFN_malloc(32768, _("FDS RAM"))))
+ {
+  return(0);
+ }
+
+ if(!(CHRRAM = (uint8*)MDFN_malloc(8192, _("CHR RAM"))))
+ {
+  return(0);
+ }
+
 
  {
   MDFNFILE *tp;
@@ -847,11 +869,11 @@ int FDSLoad(const char *name, MDFNFILE *fp, NESGameType *gt)
   unsigned int x;
   for(x=0;x<TotalSides;x++)
   {
-   diskdatao[x]=(uint8 *)MDFN_malloc(65500);
+   diskdatao[x]=(uint8 *)MDFN_malloc(65500, _("FDS Disk Data"));
    memcpy(diskdatao[x],diskdata[x],65500);
   }
 
-  if((tp=MDFN_fopen(MDFN_MakeFName(MDFNMKF_FDS,0,0).c_str(),0,"rb",0)))
+  if((tp=MDFN_fopen(MDFN_MakeFName(MDFNMKF_SAV,0, "fds").c_str(),0,"rb",0)))
   {
    FreeFDSMemory();
    if(!SubLoad(tp))
@@ -864,7 +886,7 @@ int FDSLoad(const char *name, MDFNFILE *fp, NESGameType *gt)
   }
  }
 
- MDFNGameInfo->nes.type=GIT_FDS;
+ MDFNGameInfo->GameType = GMT_DISK;
 
  SelectDisk=0;
  InDisk=255;
@@ -892,7 +914,7 @@ void FDSClose(void)
 
  if(!DiskWritten) return;
 
- if(!(fp=fopen(MDFN_MakeFName(MDFNMKF_FDS,0,0).c_str(),"wb")))
+ if(!(fp=fopen(MDFN_MakeFName(MDFNMKF_SAV, 0, "fds").c_str(),"wb")))
   return;
 
  for(x=0;x<TotalSides;x++)
