@@ -19,8 +19,6 @@
  */
 
 #include        <string.h>
-#include	<stdio.h>
-#include	<stdlib.h>
 #include	<stdarg.h>
 
 #include	"nes.h"
@@ -88,7 +86,6 @@ void FlushGenieRW(void)
   for(x=0;x<0x8000;x++)
   {
    ARead[x+0x8000]=AReadG[x];
-   NonCheatARead[x + 0x8000] = AReadG[x];
    BWrite[x+0x8000]=BWriteG[x];
   }
   free(AReadG);
@@ -117,6 +114,7 @@ void SetReadHandler(int32 start, int32 end, readfunc func, bool snc)
    func=ANull;
 
   if(RWWrap)
+  {
    for(x=end;x>=start;x--)
    {
     if(x>=0x8000)
@@ -124,10 +122,11 @@ void SetReadHandler(int32 start, int32 end, readfunc func, bool snc)
     else
     {
      ARead[x]=func;
-     if(snc)
-      NonCheatARead[x] = func;
     }
+    if(snc)
+     NonCheatARead[x] = func;
    }
+  }
   else
    for(x=end;x>=start;x--)
    {
@@ -268,17 +267,9 @@ static void CloseGame(void)
  MDFNSND_Close();
 }
 
-int UNIFLoad(const char *name, MDFNFILE *fp, NESGameType *);
-int iNESLoad(const char *name, MDFNFILE *fp, NESGameType *);
-int FDSLoad(const char *name, MDFNFILE *fp, NESGameType *);
-int NSFLoad(const char *name, MDFNFILE *fp, NESGameType *);
-
-static int Load(const char *name, MDFNFILE *fp)
+static void InitCommon(const char *name)
 {
-	int (*LoadFunctions[4])(const char *name, MDFNFILE *fp, NESGameType *) = { iNESLoad, UNIFLoad, NSFLoad, FDSLoad };
-	int x;
-
-	NESIsVSUni = FALSE;
+        NESIsVSUni = FALSE;
         PPU_hook = 0;
         GameHBIRQHook = 0;
 
@@ -287,24 +278,25 @@ static int Load(const char *name, MDFNFILE *fp)
         PAL &= 1;
         MDFN_SetPPUPalette(0);
 
-	MDFNGameInfo->GameType = GMT_CART;
+        MDFNGameInfo->GameType = GMT_CART;
         MDFNGameInfo->VideoSystem = VIDSYS_NONE;
 
         MDFNGameInfo->cspecial = NULL;
-	MDFNGameInfo->soundchan = 1;
-	MDFNGameInfo->GameSetMD5Valid = FALSE;
+        MDFNGameInfo->soundchan = 1;
+        MDFNGameInfo->GameSetMD5Valid = FALSE;
+
 
         if(MDFN_GetSettingB("nes.fnscan"))
-	{
-	 if(strstr(name, "(U)"))
-	  MDFNGameInfo->VideoSystem = VIDSYS_NTSC;
+        {
+         if(strstr(name, "(U)"))
+          MDFNGameInfo->VideoSystem = VIDSYS_NTSC;
          else if(strstr(name, "(J)"))
           MDFNGameInfo->VideoSystem = VIDSYS_NTSC;
          else if(strstr(name, "(E)") || strstr(name, "(G)"))
           MDFNGameInfo->VideoSystem = VIDSYS_PAL;
         }
 
-	GameInterface = (NESGameType *)calloc(1, sizeof(NESGameType));
+        GameInterface = (NESGameType *)calloc(1, sizeof(NESGameType));
 
         SetReadHandler(0x0000, 0xFFFF, ANull);
         SetWriteHandler(0x0000, 0xFFFF, BNull);
@@ -320,38 +312,50 @@ static int Load(const char *name, MDFNFILE *fp)
 
         MDFNMP_Init(1024, 65536 / 1024);
 
-	#ifdef WANT_DEBUGGER
-        MDFNNES_ResetASpaceInfo();
-	MDFNNES_ResetRegGroupsInfo();
+        #ifdef WANT_DEBUGGER
+        MDFNDBG_AddRegGroup(&NESCPURegsGroup);
+        MDFNDBG_AddRegGroup(&NESPPURegsGroup);
 
-	MDFNNES_AddRegGroup(&NESCPURegsGroup);
-	MDFNNES_AddRegGroup(&NESPPURegsGroup);
+        MDFNDBG_AddASpace(NESDBG_GetAddressSpaceBytes, NESDBG_PutAddressSpaceBytes, "cpu", "CPU", 16);
+        MDFNDBG_AddASpace(NESPPU_GetAddressSpaceBytes, NESPPU_PutAddressSpaceBytes, "ppu", "PPU", 14);
+        #endif
+}
 
-        MDFNNES_AddASpace(NESDBG_GetAddressSpaceBytes, NESDBG_PutAddressSpaceBytes, "cpu", "CPU", 16);
-        MDFNNES_AddASpace(NESPPU_GetAddressSpaceBytes, NESPPU_PutAddressSpaceBytes, "ppu", "PPU", 14);
-	#endif
+bool UNIFLoad(const char *name, MDFNFILE *fp, NESGameType *);
+bool iNESLoad(const char *name, MDFNFILE *fp, NESGameType *);
+bool FDSLoad(const char *name, MDFNFILE *fp, NESGameType *);
+bool NSFLoad(const char *name, MDFNFILE *fp, NESGameType *);
 
-	for(x=0; x<4; x++)
-	{
-	 int t = LoadFunctions[x](name, fp, GameInterface);
+bool iNES_TestMagic(const char *name, MDFNFILE *fp);
+bool UNIF_TestMagic(const char *name, MDFNFILE *fp);
+bool FDS_TestMagic(const char *name, MDFNFILE *fp);
+bool NSF_TestMagic(const char *name, MDFNFILE *fp);
 
-	 if(t == 0)
-         {
-	  free(GameInterface);
-	  GameInterface = NULL;
-	  return(0);
-	 }
-	 else if(t == -1)
+static int Load(const char *name, MDFNFILE *fp)
+{
+	bool (*MagicFunctions[4])(const char *name, MDFNFILE *fp) = { iNES_TestMagic, UNIF_TestMagic, NSF_TestMagic, FDS_TestMagic };
+	bool (*LoadFunctions[4])(const char *name, MDFNFILE *fp, NESGameType *) = { iNESLoad, UNIFLoad, NSFLoad, FDSLoad };
+	bool (*LoadFunction)(const char *name, MDFNFILE *fp, NESGameType *) = NULL;
+	int x;
+
+	for(x = 0; x < 4; x++)
+	 if(MagicFunctions[x](name, fp))
 	 {
-	  if(x == 3)
-	  {
-	   free(GameInterface);
-	   GameInterface = NULL;
-           return(-1);
-	  }
+	  LoadFunction = LoadFunctions[x];
+	  break;
 	 }
-	 else
-	  break;	// File loaded successfully.
+
+	// If the file type isn't recognized, return -1!
+	if(!LoadFunction)
+	 return(-1);
+
+	InitCommon(name);
+
+	if(!LoadFunction(name, fp, GameInterface))
+	{
+	 free(GameInterface);
+	 GameInterface = NULL;
+	 return(0);
 	}
 
 	if(MDFNGameInfo->GameType != GMT_PLAYER)
@@ -396,50 +400,37 @@ void MDFNNES_Kill(void)
  MDFN_KillGenie();
 }
 
-static int needrew = 0;
-
-static void DoRewind(void)
+static void Emulate(EmulateSpecStruct *espec)
 {
- needrew = 1;
-}
+ int ssize;
 
-static void Emulate(uint32 *pXBuf, MDFN_Rect *LineWidths, int16 **SoundBuf, int32 *SoundBufSize, int skip)
-{
- int r,ssize;
- int didrew;
-
- MDFNGameInfo->fb = pXBuf;
+ MDFNGameInfo->fb = espec->pixels;
 
  MDFN_UpdateInput();
 
- if(MDFNGameInfo->GameType != GMT_PLAYER)
-  didrew = MDFN_StateEvil(needrew); 
- else
-  didrew = 0;
-
  if(geniestage!=1) MDFNMP_ApplyPeriodicCheats();
- r=MDFNPPU_Loop(skip);
+ MDFNPPU_Loop(espec->skip);
 
- ssize=FlushEmulateSound(didrew);
- needrew = 0;
+ ssize = FlushEmulateSound(espec->NeedSoundReverse);
+ espec->NeedSoundReverse = 0;
 
  timestampbase += timestamp;
 
  timestamp = 0;
 
  if(MDFNGameInfo->GameType == GMT_PLAYER)
-  MDFNNES_DrawNSF(pXBuf, WaveFinal, ssize);
+  MDFNNES_DrawNSF(espec->pixels, WaveFinal, ssize);
 
- *SoundBuf=WaveFinal;
- *SoundBufSize=ssize;
+ *(espec->SoundBuf) = WaveFinal;
+ *(espec->SoundBufSize) = ssize;
 
 
  if(MDFNGameInfo->GameType != GMT_PLAYER)
  {
   if(NESIsVSUni)
-   MDFN_VSUniDraw(pXBuf);
+   MDFN_VSUniDraw(espec->pixels);
  }
- MDFN_DrawInput(pXBuf);
+ MDFN_DrawInput(espec->pixels);
 }
 
 void ResetNES(void)
@@ -499,6 +490,39 @@ void PowerNES(void)
 
 static int StateAction(StateMem *sm, int load, int data_only)
 {
+ #if 0
+ uint8 new_geniestage = geniestage;
+
+ SFORMAT StateRegs[] =
+ {
+  SFVAR(new_geniestage),
+  SFEND
+ };
+
+ if(!MDFNSS_StateAction(sm, load, data_only, StateRegs, "MAIN"))
+  return(0);
+
+ if(geniestage)
+ {
+  if(geniestage == 1 && new_geniestage == 2)
+  {
+
+  }
+ }
+ else if(new_geniestage == 1) // If Game Genie emulation is disabled, and we're trying to load a save-state where we're
+			      // in the Game Genie screen, error out!
+ {
+  return(0);
+ }
+ #endif
+
+ if(geniestage == 1)
+ {
+  if(!data_only)
+   MDFN_DispMessage((UTF8 *)_("Cannot use states in GG Screen."));
+  return(0);
+ }
+
  if(!X6502_StateAction(sm, load, data_only))
   return(0);
 
@@ -618,8 +642,8 @@ static DebuggerInfoStruct DBGInfo =
  3,
  16,
  16,
- 0x0000,
- 
+ 0x0000, // Default watch addr
+ 0x0000, // ZP
  NESDBG_MemPeek,
  NESDBG_Disassemble,
  NULL,
@@ -633,41 +657,6 @@ static DebuggerInfoStruct DBGInfo =
  NESPPU_SetGraphicsDecode,
  NESPPU_GetGraphicsDecodeBuffer,
 };
-
-void MDFNNES_ResetRegGroupsInfo(void)
-{
- DBGInfo.RegGroups.clear();
-}
-
-void MDFNNES_AddRegGroup(RegGroupType *groupie)
-{
- DBGInfo.RegGroups.push_back(groupie);
-}
-
-void MDFNNES_ResetASpaceInfo(void)
-{
- DBGInfo.AddressSpaces.clear();
-}
-
-void MDFNNES_AddASpace(void (*gasb)(const char *name, uint32 Address, uint32 Length, uint8 *Buffer), void (*pasb)(const char *name, uint32 Address, uint32 Length, uint32 Granularity, bool hl, const uint8 *Buffer), const char *name, const char *long_name, uint32 TotalBits, bool IsSegmented, uint32 SegmentBits, uint32 OffsetBits, uint32 BitsOverlapped)
-{
- AddressSpaceType newt;
-
- memset(&newt, 0, sizeof(newt));
-
- newt.GetAddressSpaceBytes = gasb;
- newt.PutAddressSpaceBytes = pasb;
-
- newt.name = name;
- newt.long_name = long_name;
- newt.TotalBits = TotalBits;
- newt.IsSegmented = IsSegmented;
- newt.SegmentBits = SegmentBits;
- newt.OffsetBits = OffsetBits;
- newt.BitsOverlapped = BitsOverlapped;
-
- DBGInfo.AddressSpaces.push_back(newt);
-}
 #endif
 
 
@@ -687,7 +676,6 @@ MDFNGI EmulatedNES =
  RemoveReadPatches,
  MemRead,
  StateAction,
- DoRewind,
  Emulate,
  MDFNNES_SetPixelFormat,
  MDFNNES_SetInput,
