@@ -58,6 +58,16 @@ static void RefreshSprites(void);
 static void Fixit1(void);
 static uint32 ppulut1[256], ppulut2[256], ppulut3[128];
 
+
+static void DoGfxDecode(void);
+static uint32 *GfxDecode_Buf = NULL;
+static int GfxDecode_Width = 0;
+static int GfxDecode_Height = 0;
+static int GfxDecode_Line = -1;
+static int GfxDecode_Layer = 0;
+static int GfxDecode_Scroll = 0;
+static int GfxDecode_Pbn = 0;
+
 static void makeppulut(void)
 {
  int x;
@@ -179,7 +189,7 @@ static INLINE void RefreshPaletteCacheS(unsigned int which)
 
 void MDFNNES_SetPixelFormat(int r, int g, int b)
 {
- if(MDFNGameInfo->nes.type != GIT_NSF)
+ if(MDFNGameInfo->GameType != GMT_PLAYER)
   RefreshPaletteCache();
 }
 
@@ -1160,7 +1170,7 @@ int MDFNPPU_Loop(int skip)
                                 */
    X6502_Run(12);
 
-   if(MDFNGameInfo->nes.type==GIT_NSF)
+   if(MDFNGameInfo->GameType == GMT_PLAYER)
     DoNSFFrame();
    else
    {
@@ -1175,12 +1185,16 @@ int MDFNPPU_Loop(int skip)
    /* Clean this stuff up later. */
    spork=numsprites=0;
 
-   if(MDFNGameInfo->nes.type==GIT_NSF)
+   if(MDFNGameInfo->GameType == GMT_PLAYER)
     X6502_Run((256+85)*240);
    else
    {
     for(scanline=0;scanline<240;)       //scanline is incremented in  DoLine.  Evil. :/
+    {
+     if(scanline == GfxDecode_Line)
+      DoGfxDecode();
      DoLine(skip);
+    }
     if(MMC5Hack && (ScreenON || SpriteON)) MMC5_hb(scanline);
    }
   } /* else... to if(ppudead) */
@@ -1211,6 +1225,11 @@ uint32 NESPPU_GetRegister(const std::string &name)
   return(VRAMBuffer);
  else if(name == "V-Toggle")
   return(vtoggle);
+ else if(name == "Scanline")
+ {
+  if(scanline == -1) return(261);
+  else return(scanline);
+ }
  else return(0);
 }
 
@@ -1262,6 +1281,7 @@ int MDFNPPU_StateAction(StateMem *sm, int load, int data_only)
   SFARRAYN(PALRAM, 0x20, "PRAM"),
   SFARRAYN(SPRAM, 0x100, "SPRA"),
   SFARRAYN(PPU, 0x4, "PPUR"),
+  SFVARN(BurstPhase, "BurstPhase"),
   SFVARN(kook, "KOOK"),
   SFVARN(ppudead, "DEAD"),
   SFVARN(PPUSPL, "PSPL"),
@@ -1343,7 +1363,7 @@ void MDFNPPU_Init(void)
  rendis = 0;
  maxsprites = MDFN_GetSettingB("nes.no8lim") ? 64 : 8;
 
- if(MDFN_GetSettingB("nes.ntscblitter") && !PAL && MDFNGameInfo->nes.type != GIT_VSUNI && MDFNGameInfo->nes.type != GIT_NSF)
+ if(MDFN_GetSettingB("nes.ntscblitter") && !PAL && !NESIsVSUni && MDFNGameInfo->GameType != GMT_PLAYER)
  {
   static float matrix[6];
 
@@ -1391,33 +1411,187 @@ void MDFNPPU_Init(void)
  else
   scanlines_per_frame=262;
 
- RedoRL();
+ if(MDFNGameInfo->GameType != GMT_PLAYER)
+  RedoRL();
 }
 
-
-// If hl is true, do not cause any change in the underlying hardware state.
-uint32 NESPPU_GfxMemPeek(uint32 A, unsigned int bsize, bool hl)
+void NESPPU_SettingChanged(const char *name)
 {
- A &= 0x3FFF;
- uint32 ret = 0;
-
- for(int n = 0; n < bsize; n++)
+ if(!strcmp(name, "nes.no8lim"))
  {
-  if(A < 0x2000)
-   ret |= VPage[A >> 10][A] << (n * 8);
-  else if(A >= 0x3f00)
-   ret |= PALRAM[A & ((A & 3)? 0x1F : 0x0C)];
-  else
-   ret |= vnapage[(A >> 10) & 0x3][A & 0x3FF];
-
-  A = (A + 1) & 0x3FFF;
+  maxsprites = MDFN_GetSettingB("nes.no8lim") ? 64 : 8;
  }
- return(ret);
 }
 
- // If hl is true, write to the underlying ROM or RAM directly, if present
-void NESPPU_GfxMemPoke(uint32 A, uint32 V, unsigned int bsize, bool hl)
+void NESPPU_GetAddressSpaceBytes(const char *name, uint32 Address, uint32 Length, uint8 *Buffer)
 {
+ if(!strcmp(name, "ppu"))
+ {
+  while(Length--)
+  {
+   Address &= 0x3FFF;
 
+   if(Address < 0x2000)
+    *Buffer = VPage[Address >> 10][Address];
+   else if(Address >= 0x3f00)
+    *Buffer = PALRAM[Address & ((Address & 3)? 0x1F : 0x0C)];
+   else
+    *Buffer = vnapage[(Address >> 10) & 0x3][Address & 0x3FF];
+
+   Address++;
+   Buffer++;
+  }
+ }
 }
 
+void NESPPU_PutAddressSpaceBytes(const char *name, uint32 Address, uint32 Length, uint32 Granularity, bool hl, const uint8 *Buffer)
+{
+ if(!strcmp(name, "ppu"))
+ {
+  while(Length--)
+  {
+   Address &= 0x3FFF;
+
+   if(Address < 0x2000)
+    VPage[Address >> 10][Address] = *Buffer;
+   else if(Address >= 0x3f00)
+   {
+    uint32 tmp = Address & ((Address & 3)? 0x1F : 0x0C);
+    PALRAM[tmp] = PALRAMCache[tmp] = *Buffer;
+    RefreshPaletteCacheS(tmp);
+   }
+   else
+    vnapage[(Address >> 10) & 0x3][Address & 0x3FF] = *Buffer;
+
+   Address++;
+   Buffer++;
+  }
+ }
+}
+
+
+void NESPPU_SetGraphicsDecode(int line, int which, int w, int h, int xscroll, int yscroll, int pbn)
+{
+ if(line == -1)
+ {
+  if(GfxDecode_Buf)
+  {
+   free(GfxDecode_Buf);
+   GfxDecode_Buf = NULL;
+  }
+ }
+ else
+  GfxDecode_Buf = (uint32*)realloc(GfxDecode_Buf, w * h * sizeof(uint32) * 3); // *2 for extra address info.
+
+ GfxDecode_Line = line;
+ GfxDecode_Width = w;
+ GfxDecode_Height = h;
+ GfxDecode_Layer = which;
+ GfxDecode_Scroll = yscroll;
+ GfxDecode_Pbn = pbn;
+
+ if(GfxDecode_Line == 0xB00B13)
+  DoGfxDecode();
+}
+
+uint32 *NESPPU_GetGraphicsDecodeBuffer(void)
+{
+ return(GfxDecode_Buf);
+}
+
+static void DoGfxDecode(void)
+{
+ uint32 *target = GfxDecode_Buf;
+ int pbn = GfxDecode_Pbn & 0x3;
+ uint32 neo_palette[4];
+
+ if(GfxDecode_Pbn == -1)
+ {
+  for(int x = 0; x <  4; x++)
+   neo_palette[x] = MK_COLORA(x * 85, x * 85, x * 85, 0xFF);
+ }
+ else
+  for(int x = 0; x < 4; x++)
+   neo_palette[x] = PALRAMLUTCache[PALRAMCache[pbn * 4 + x] & 0x3F] | MK_COLORA(0, 0, 0, 0xFF);
+
+  for(int y = 0; y < GfxDecode_Height; y++)
+  {
+   for(int x = 0; x < GfxDecode_Width; x+=8)
+   {
+    int which_tile = (x / 8) + (GfxDecode_Scroll + (y / 8)) * (GfxDecode_Width / 8);
+
+    uint8 *cg_ptr;
+    uint8 cg[2];
+    int tile_c = 0;
+
+    if(MMC5Hack)
+    {
+     if(GfxDecode_Layer) // Sprites
+     {
+      //which_tile &= 0x1FF;
+      cg_ptr = MMC5SPRVRAMADR(which_tile * 16);
+     }
+     else
+     {
+      switch(MMC5HackCHRMode)
+      {
+       default: //which_tile &= 0x1FF;
+                cg_ptr = MMC5BGVRAMADR(which_tile * 16);
+		break;
+      }
+     }
+    }
+    else
+    {
+     if(GfxDecode_Layer) // Sprites
+     {
+      pbn |= 0x4;
+      if(Sprite16)
+      {
+       tile_c = 0x200;
+       cg_ptr = VRAMADR(which_tile * 16);
+      }
+      else
+      {
+       tile_c = 0x100;
+
+       if(SpAdrHI)
+        cg_ptr = VRAMADR(0x1000 + which_tile * 16);
+       else
+        cg_ptr = VRAMADR(0x0000 + which_tile * 16);
+      }
+     }
+     else // Background
+     {
+      tile_c = 0x100;
+      if(BGAdrHI)
+       cg_ptr = VRAMADR(0x1000 + which_tile * 16);
+      else
+       cg_ptr = VRAMADR(0x0000 + which_tile * 16);
+     }
+    }
+
+    if(which_tile >= tile_c)
+    {
+     for(int sx = 0; sx < 8; sx++) target[x + sx] = MK_COLORA(0, 0, 0, 0);
+     continue;
+    }
+
+    cg[0] = cg_ptr[0 + (y & 0x7)];
+    cg[1] = cg_ptr[8 + (y & 0x7)];
+
+    for(int sx = 0; sx < 8; sx++)
+     target[x + sx] = neo_palette[((cg[0] >> (7-sx)) & 0x1) | (((cg[1] >> (7-sx)) & 0x1) << 1)];
+
+    target[x + GfxDecode_Width*2 + 0] = target[x + GfxDecode_Width*2 + 1] = target[x + GfxDecode_Width*2 + 2] = target[x + GfxDecode_Width*2 + 3] =
+    target[x + GfxDecode_Width*2 + 4] = target[x + GfxDecode_Width*2 + 5] = target[x + GfxDecode_Width*2 + 6] = target[x + GfxDecode_Width*2 + 7] = which_tile * 16;
+
+    target[x + GfxDecode_Width*1 + 0]=target[x + GfxDecode_Width*1 + 1]=target[x + GfxDecode_Width*1 + 2]=target[x + GfxDecode_Width*1 + 3] =
+    target[x + GfxDecode_Width*1 + 4]=target[x + GfxDecode_Width*1 + 5]=target[x + GfxDecode_Width*1 + 6]=target[x + GfxDecode_Width*1 + 7] = which_tile;
+   }
+   target += GfxDecode_Width * 3;
+  }
+
+
+
+}
