@@ -94,11 +94,16 @@ static const uint8 lengthtable[0x20]=
  0x0A, 0xFE, 0x14, 0x02, 0x28, 0x04, 0x50, 0x06, 0xa0, 0x08, 0x3c, 0x0a, 0x0e, 0x0c, 0x1a, 0x0e, 0x0c, 0x10, 0x18, 0x12, 0x30, 0x14, 0x60, 0x16, 0xc0, 0x18, 0x48, 0x1a, 0x10, 0x1c, 0x20, 0x1E
 };
 
-static const uint32 NoiseFreqTable[0x10]=
+static const uint32 NTSCNoiseFreqTable[0x10]=
 {
  4, 8, 16, 32, 64, 96, 128, 160, 202, 254, 380, 508, 762, 1016, 2034, 4068
 };
 
+static const uint32 PALNoiseFreqTable[0x10] =
+{
+ 4, 7, 14, 30, 60, 88, 118, 148, 188, 236, 354, 472, 708, 944, 1890, 3778
+};
+
 static const uint32 NTSCDMCTable[0x10]=
 {
  428,380,340,320,286,254,226,214,
@@ -107,8 +112,7 @@ static const uint32 NTSCDMCTable[0x10]=
 
 static const uint32 PALDMCTable[0x10]=
 {
- 397, 353, 315, 297, 265, 235, 209, 198, 
- 176, 148, 131, 118, 98, 78, 66, 50, 
+ 398, 354, 316, 298, 276, 236, 210, 198, 176, 148, 132, 118, 98, 78, 66, 50
 };
 
 // $4010        -        Frequency
@@ -675,7 +679,13 @@ static void RDoNoise(void)
 
  outo=amptab[nreg & 1];
 
- int32 wl = NoiseFreqTable[PSG[0xE]&0xF];
+
+ int32 wl;
+
+ if(SoundPAL)
+  wl = PALNoiseFreqTable[PSG[0xE]&0xF];
+ else
+  wl = NTSCNoiseFreqTable[PSG[0xE]&0xF];
 
  if(PSG[0xE]&0x80)        // "short" noise
   for(V=ChannelBC[3];V<SOUNDTS;V++)
@@ -720,6 +730,7 @@ static void SetNESSoundMap(void)
 
 static double capacimoo = 0;
 static int32 inbuf=0;
+
 int FlushEmulateSound(int reverse)
 {
   int x;
