@@ -303,7 +303,6 @@ void ADPCM_Reset()
 	LastCmd		= 0;
 
 	// for adpcm play
-	SampleFreq = 0;
 	CurrentVolume = InitialVolume;
 	VolumeStep = InitialVolume / 10;
 	bFadeIn  = FALSE;
@@ -333,6 +332,7 @@ void ADPCM_Power(void)
  ClockCount = 0;
  FadeCycle = 0;
  LastCmd = 0;
+ SampleFreq = 0;
 
  ad_sample = 2048;
  ad_ref_index = 0;
@@ -430,15 +430,22 @@ void ADPCM_Write180D(uint8 data)
 	 CacheLengthCount = LengthCount = Addr << 1;
 	}
 
-	if(data & 0x08)
-	{
-         ReadAddr = ((Addr - 1) << 1) & 0x1FFFF;
-	}
+        // D2 and D3 control read address
+        if(!(LastCmd & 0x8) && (data & 0x08))
+        {
+         if(data & 0x4)
+          ReadAddr = Addr << 1;
+         else
+          ReadAddr = ((Addr - 1) << 1) & 0x1FFFF;
+        }
 
-	if((data & 0x03) == 0x03)
-	{
+        // D0 and D1 control write address
+        if(!(LastCmd & 0x2) && (data & 0x2))
+        {
          WriteAddr = Addr;
-	}
+         if(!(data & 0x1))
+          WriteAddr = (WriteAddr - 1) & 0xFFFF;
+        }
 	LastCmd = data;
 }
 
