@@ -23,4 +23,12 @@ void PCEDBG_CheckBP(int type, uint32 address, unsigned int len);
 void PCEDBG_GetAddressSpaceBytes(const char *name, uint32 Address, uint32 Length, uint8 *Buffer);
 void PCEDBG_PutAddressSpaceBytes(const char *name, uint32 Address, uint32 Length, uint32 Granularity, bool hl, const uint8 *Buffer);
 
+
+void PCEDBG_SetLogFunc(void (*func)(const char *, const char *));
+
+void PCEDBG_DoLog(const char *type, const char *format, ...);
+char *PCEDBG_ShiftJIS_to_UTF8(const uint16 sjc);
+
+extern bool PCE_LoggingOn;
+
 #endif
