@@ -19,12 +19,13 @@
 
 #include <errno.h>
 #include <string.h>
+
 #include "pce.h"
 #include "cdrom.h"
 #include "adpcm.h"
 #include "../md5.h"
 #include "../file.h"
-#include "../cdromif.h"
+#include "../cdrom/cdromif.h"
 #include "../mempatcher.h"
 
 static uint8 *HuCROM;
@@ -32,7 +33,7 @@ static uint8 HuCROMMask;
 static uint8 *ROMMap[0x100];
 
 static bool IsPopulous;
-static bool IsTsushin;
+bool IsTsushin;
 bool PCE_IsCD;
 
 static uint8 *TsushinRAM = NULL; // 0x8000
@@ -97,10 +98,17 @@ DECLFR(PCE_ACRead)
 	 case 0x1:
 		if(ACRAMUsed)
 		{
-	 	 if(port->control & 0x2)
-		  ret = ACRAM[(port->base + port->offset) & 0x1fffff];
-		 else
-		  ret = ACRAM[port->base & 0x1fffff];
+                 uint32 aci;
+
+                 aci = port->base;
+                 if(port->control & 0x2)
+                 {
+                  aci += port->offset;
+                  if(port->control & 0x8)
+                   aci += 0xFF0000;
+                 }
+                 aci &= 0x1FFFFF;
+		 ret = ACRAM[aci];
 		}
 		else
 		 ret = 0;
@@ -170,10 +178,20 @@ DECLFW(PCE_ACWrite)
 				  exit(1); // FIXME!  
 				 }
 				}
-                                if (port->control & 2)
-                                        ACRAM[(port->base + port->offset) & 0x1fffff] = V;
-                                else
-                                        ACRAM[port->base & 0x1fffff] = V;
+
+				{
+                 		 uint32 aci;
+
+                 		 aci = port->base;
+		                 if(port->control & 0x2)
+		                 {
+		                  aci += port->offset;
+		                  if(port->control & 0x8)
+		                   aci += 0xFF0000;
+		                 }
+		                 aci &= 0x1FFFFF;
+		                 ACRAM[aci] = V;
+				}
                                 increment_acaddr(port);
                                 return;
 
@@ -194,11 +212,12 @@ DECLFW(PCE_ACWrite)
 
                                 if ((port->control & 0x60) == 0x40)
                                 {
-                                        if (port->control & 0x08)
-                                                port->base += port->offset + 0xff0000;
-                                        else
-                                                port->base += port->offset;
-                                        port->base &= 0xffffff;
+                                 port->base += port->offset;
+
+			         if(port->control & 0x08)
+			          port->base += 0xFF0000;
+
+                                 port->base &= 0xffffff;
                                 }
                                 return;
                         case 0x7:
@@ -209,13 +228,18 @@ DECLFW(PCE_ACWrite)
                                 return;
                         case 0x9:
                                 port->control = V & 0x7f;            // D7 is not used
+				//if((V & 0x0C)) printf("%02x\n", V);
                                 return;
                         case 0xa:
                                 // value written is not used
                                 if ((port->control & 0x60) == 0x60)
                                 {
-                                        port->base += port->offset;
-                                        port->base &= 0xffffff;
+                                 port->base += port->offset;
+
+			         if(port->control & 0x08)
+			          port->base += 0xFF0000;
+
+                                 port->base &= 0xffffff;
                                 }
                                 return;
                 }
@@ -244,7 +268,7 @@ static DECLFR(CDRAMRead)
 
 static DECLFR(SaveRAMRead)
 {
- if(!PCE_IsCD || PCECD_IsBRAMEnabled() && (A & 8191) < 2048)
+ if((!PCE_IsCD || PCECD_IsBRAMEnabled()) && (A & 8191) < 2048)
   return(SaveRAM[A & 2047]);
  else
   return(0xFF);
@@ -252,7 +276,7 @@ static DECLFR(SaveRAMRead)
 
 static DECLFW(SaveRAMWrite)
 {
- if(!PCE_IsCD || PCECD_IsBRAMEnabled() && (A & 8191) < 2048)
+ if((!PCE_IsCD || PCECD_IsBRAMEnabled()) && (A & 8191) < 2048)
   SaveRAM[A & 2047] = V;
 }
 
@@ -355,7 +379,7 @@ int HuCLoad(uint8 *data, uint32 len, uint32 crc32)
   
   if(!(PopRAM = (uint8 *)MDFN_malloc(32768, _("Populous RAM"))))
   {
-   free(HuCROM);
+   MDFN_free(HuCROM);
    return(0);
   }
   memset(PopRAM, 0xFF, 32768);
@@ -380,7 +404,7 @@ int HuCLoad(uint8 *data, uint32 len, uint32 crc32)
 
   if(!(TsushinRAM = (uint8*)MDFN_malloc(0x8000, _("Tsushin Booster RAM"))))
   {
-   free(HuCROM);
+   MDFN_free(HuCROM);
    return(0);
   }
   memset(TsushinRAM, 0xFF, 0x8000);
@@ -496,13 +520,13 @@ int HuCLoadCD(const char *bios_path)
   for(int x = 0; x < 128; x++)
   {
    memset(sector_buffer, 0, 2048);
-   CDIF_ReadSector(sector_buffer, NULL, NULL, start_sector + x, 1);
+   CDIF_ReadSector(sector_buffer, NULL, start_sector + x, 1);
    md5.update(sector_buffer, 2048);
   }
  }
  md5.finish(MDFNGameInfo->MD5);
  MDFN_printf(_("CD MD5(first 256KiB):   0x%s\n"), md5_context::asciistr(MDFNGameInfo->MD5, 0).c_str());
-
+ MDFN_printf(_("Arcade Card Emulation:  %s\n"), PCE_ACEnabled ? _("Enabled") : _("Disabled")); 
  for(int x = 0; x < 0x40; x++)
  {
   ROMMap[x] = HuCPUFastMap[x] = &HuCROM[(x & 0x1F) * 8192] - x * 8192;
@@ -511,7 +535,7 @@ int HuCLoadCD(const char *bios_path)
 
  if(!(CDRAM = (uint8 *)MDFN_malloc(262144, _("CD RAM"))))
  {
-  free(HuCROM);
+  MDFN_free(HuCROM);
   return(0);
  }
 
@@ -523,11 +547,14 @@ int HuCLoadCD(const char *bios_path)
  }
  MDFNMP_AddRAM(262144, 0x68 * 8192, CDRAM);
 
- for(int x = 0x40; x < 0x50; x++)
+ if(PCE_ACEnabled)
  {
-  ROMMap[x] = HuCPUFastMap[x] = NULL;
-  PCERead[x] = ACPhysRead;
-  PCEWrite[x] = ACPhysWrite;
+  for(int x = 0x40; x < 0x50; x++)
+  {
+   ROMMap[x] = HuCPUFastMap[x] = NULL;
+   PCERead[x] = ACPhysRead;
+   PCEWrite[x] = ACPhysWrite;
+  }
  }
 
  gzFile srp;
@@ -563,32 +590,36 @@ int HuC_StateAction(StateMem *sm, int load, int data_only)
  if(load)
   HuCSF2Latch &= 0x3;
 
- if(ACRAM && !ACRAMUsed)
+ if(PCE_ACEnabled)
  {
-  free(ACRAM);
-  ACRAM = NULL;
- }
- else if(!ACRAM && ACRAMUsed)
- {
-  if(!(ACRAM = (uint8*)MDFN_malloc(0x200000, _("Arcade Card RAM"))))
+  if(ACRAM && !ACRAMUsed)
   {
-   exit(1); // FIXME!
+   MDFN_free(ACRAM);
+   ACRAM = NULL;
+  }
+  else if(!ACRAM && ACRAMUsed)
+  {
+   if(!(ACRAM = (uint8*)MDFN_malloc(0x200000, _("Arcade Card RAM"))))
+   {
+    exit(1); // FIXME!
+   }
   }
- }
 
- SFORMAT ACStateRegs[] =
- {
-  SFVAR(AC[0].control), SFVAR(AC[0].base), SFVAR(AC[0].offset), SFVAR(AC[0].increment),
-  SFVAR(AC[1].control), SFVAR(AC[1].base), SFVAR(AC[1].offset), SFVAR(AC[1].increment),
-  SFVAR(AC[2].control), SFVAR(AC[2].base), SFVAR(AC[2].offset), SFVAR(AC[2].increment),
-  SFVAR(AC[3].control), SFVAR(AC[3].base), SFVAR(AC[3].offset), SFVAR(AC[3].increment),
-  SFVAR(ACShiftBits),
-  SFVAR(ACShift),
-  SFVAR(AC1ae5),
-  SFARRAY(ACRAM, ACRAMUsed ? 0x200000 : 0x0),
-  SFEND
- };
- ret &= MDFNSS_StateAction(sm, load, data_only, ACStateRegs, "ACRD");
+  SFORMAT ACStateRegs[] =
+  {
+   SFVAR(AC[0].control), SFVAR(AC[0].base), SFVAR(AC[0].offset), SFVAR(AC[0].increment),
+   SFVAR(AC[1].control), SFVAR(AC[1].base), SFVAR(AC[1].offset), SFVAR(AC[1].increment),
+   SFVAR(AC[2].control), SFVAR(AC[2].base), SFVAR(AC[2].offset), SFVAR(AC[2].increment),
+   SFVAR(AC[3].control), SFVAR(AC[3].base), SFVAR(AC[3].offset), SFVAR(AC[3].increment),
+   SFVAR(ACShiftBits),
+   SFVAR(ACShift),
+   SFVAR(AC1ae5),
+   SFARRAY(ACRAM, ACRAMUsed ? 0x200000 : 0x0),
+   SFEND
+  };
+  ret &= MDFNSS_StateAction(sm, load, data_only, ACStateRegs, "ACRD");
+ } // end if(PCE_ACEnabled)
+
  if(PCE_IsCD)
  {
   ret &= ADPCM_StateAction(sm, load, data_only);
@@ -603,13 +634,8 @@ void HuCClose(void)
  {
   if(PopRAM)
   {
-   gzFile fp;
-   if((fp = gzopen(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav").c_str(), "wb")))
-   {
-    gzwrite(fp, PopRAM, 32768);
-    gzclose(fp);
-   }
-   free(PopRAM);
+   MDFN_DumpToFile(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav").c_str(), 6, PopRAM, 32768);
+   MDFN_free(PopRAM);
    PopRAM = NULL;
   }
  }
@@ -617,35 +643,25 @@ void HuCClose(void)
  {
   if(TsushinRAM)
   {
-   gzFile fp;
-   if((fp = gzopen(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav").c_str(), "wb")))
-   {
-    gzwrite(fp, TsushinRAM, 32768);
-    gzclose(fp);
-   }
-   free(TsushinRAM);
+   MDFN_DumpToFile(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav").c_str(), 6, TsushinRAM, 32768);
+   MDFN_free(TsushinRAM);
    TsushinRAM = NULL;
   }
  }
  else if(IsBRAMUsed())
  {
-  gzFile fp;
-  if((fp = gzopen(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav").c_str(), "wb6")))
-  {
-   gzwrite(fp, SaveRAM, 2048);
-   gzclose(fp);
-  }
+  MDFN_DumpToFile(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav").c_str(), 6, SaveRAM, 2048);
  }
 
  if(CDRAM)
  {
-  free(CDRAM);
+  MDFN_free(CDRAM);
   CDRAM = NULL;
  }
 
  if(ACRAM)
  {
-  free(ACRAM);
+  MDFN_free(ACRAM);
   ACRAM = NULL;
  }
 
