@@ -841,7 +841,22 @@ void HuC6280_Run(int32 cycles)
 
 uint8 HuC6280_TimerRead(unsigned int A)
 {
+ #if 0
  return(HuCPU.timer_value | (PCEIODataBuffer & 0x80));
+ #endif
+
+ uint8 tvr = HuCPU.timer_value;
+ int32 time_moo = HuCPU.timer_div - _tcount;
+
+ if(time_moo <= -1 && HuCPU.timer_status)
+ {
+  if(time_moo == -1)
+   tvr = (tvr - 1) & 0x7F;
+  else
+   tvr = HuCPU.timer_load;
+ }
+
+ return(tvr | (PCEIODataBuffer & 0x80));
 }
 
 void HuC6280_TimerWrite(unsigned int A, uint8 V)
