@@ -52,6 +52,8 @@ typedef struct {
 #define SFVARN(x, n) { &x, sizeof(x) | MDFNSTATE_RLSB, n }
 
 #define SFARRAY(x, l) { x, l, #x }
+//TODO:#define SFARRAY(x, l) { x, (l * sizeof(x[0])) | ((sizeof(x[0]) == 2) ? MDFNSTATE_RLSB16) | ((sizeof(x[0]) == 4) ? MDFNSTATE_RLSB32), #x }
+
 #define SFARRAYN(x, l, n) { x, l, n }
 
 #define SFARRAY16(x, l) {x, MDFNSTATE_RLSB16 | (l * sizeof(uint16)), #x }
@@ -84,9 +86,6 @@ class SSDescriptor
  bool optional;
 };
 
-bool MDFNSS_WriteMovieJoyChunk(StateMem *st, gzFile fp, SFORMAT *sf);
-bool MDFNSS_ReadMovieJoyChunk(StateMem *st, gzFile fp, SFORMAT *sf);
-
 int MDFNSS_StateAction(StateMem *st, int load, int data_only, std::vector <SSDescriptor> &sections);
 int MDFNSS_StateAction(StateMem *st, int load, int data_only, SFORMAT *sf, const char *name, bool optional = 0);
 
@@ -96,7 +95,4 @@ void MDFN_StateEvilBegin(void);
 void MDFN_StateEvilEnd(void);
 int MDFN_StateEvil(int);
 
-bool MDFN_DrawSaveStates(uint32 *XBuf, const MDFN_Rect *OSDRect, uint32 pitch32);
-
-
 #endif
