@@ -132,8 +132,6 @@ typedef struct
 #define TRUE 1
 #endif
 
-#include <assert.h>
-
 #undef require
 #define require( expr ) assert( expr )
 
@@ -181,12 +179,18 @@ typedef struct
 
 // Some compilers' optimizers and some platforms might fubar the generated code from these macros, so some tests are run in...tests.cpp
 #define sign_9_to_s16(_value)  (((int16)((unsigned int)(_value) << 7)) >> 7)
-#define sign_10_to_s16(_value)  (((int16)((unsigned int)(_value) << 6)) >> 6)
-#define sign_11_to_s16(_value)  (((int16)((unsigned int)(_value) << 5)) >> 5)
-#define sign_12_to_s16(_value)  (((int16)((unsigned int)(_value) << 4)) >> 4)
-#define sign_13_to_s16(_value)  (((int16)((unsigned int)(_value) << 3)) >> 3)
-#define sign_14_to_s16(_value)  (((int16)((unsigned int)(_value) << 2)) >> 2)
-#define sign_15_to_s16(_value)  (((int16)((unsigned int)(_value) << 1)) >> 1)
+#define sign_10_to_s16(_value)  (((int16)((uint32)(_value) << 6)) >> 6)
+#define sign_11_to_s16(_value)  (((int16)((uint32)(_value) << 5)) >> 5)
+#define sign_12_to_s16(_value)  (((int16)((uint32)(_value) << 4)) >> 4)
+#define sign_13_to_s16(_value)  (((int16)((uint32)(_value) << 3)) >> 3)
+#define sign_14_to_s16(_value)  (((int16)((uint32)(_value) << 2)) >> 2)
+#define sign_15_to_s16(_value)  (((int16)((uint32)(_value) << 1)) >> 1)
 
+// This obviously won't convert higher-than-32 bit numbers to signed 32-bit ;)
+// Also, this shouldn't be used for 8-bit and 16-bit signed numbers, since you can
+// convert those faster with typecasts...
+#define sign_x_to_s32(_bits, _value) (((int32)((uint32)(_value) << (32 - _bits))) >> (32 - _bits))
+
+#include "error.h"
 
 #endif
